@@ -1,54 +1,42 @@
-/* 
-   HCIDump - HCI packet analyzer	
-   Copyright (C) 2000-2001 Maxim Krasnyansky <maxk@qualcomm.com>
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License version 2 as
-   published by the Free Software Foundation;
-
-   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
-   IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY CLAIM,
-   OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER
-   RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
-   NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE
-   USE OR PERFORMANCE OF THIS SOFTWARE.
-
-   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, COPYRIGHTS,
-   TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS SOFTWARE IS DISCLAIMED.
-*/
-
 /*
-	SDP parser.
-	Copyright (C) 2001 Ricky Yuen <ryuen@qualcomm.com>
-*/
-
-/*
- * $Id: sdp.c,v 1.14 2002/11/06 23:10:55 holtmann Exp $
+ *
+ *  Bluetooth packet analyzer - SDP parser
+ *
+ *  Copyright (C) 2001-2002  Ricky Yuen <ryuen@qualcomm.com>
+ *  Copyright (C) 2003-2004  Marcel Holtmann <marcel@holtmann.org>
+ *
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ *
+ *  $Id: sdp.c,v 1.31 2004/09/13 19:02:57 holtmann Exp $
  */
 
 #include <stdio.h>
+#include <errno.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <errno.h>
 #include <string.h>
 
-#include <sys/socket.h>
 #include <sys/types.h>
-#include <asm/types.h> 
 #include <netinet/in.h>
 
-#include <bluetooth/bluetooth.h>
-#include <bluetooth/hci.h>
-#include <bluetooth/l2cap.h>
-
 #include "parser.h"
 #include "sdp.h"
 
-static inline void print_de(int, struct frame*, int *split);
-
-sdp_siz_idx_lookup_table_t sdp_siz_idx_lookup_table[] = {
+static sdp_siz_idx_lookup_table_t sdp_siz_idx_lookup_table[] = {
 	{ 0, 1  }, /* Size index = 0 */
 	{ 0, 2  }, /*              1 */
 	{ 0, 4  }, /*              2 */
@@ -59,7 +47,7 @@ sdp_siz_idx_lookup_table_t sdp_siz_idx_lookup_table[] = {
 	{ 1, 4  }, /*              7 */
 };
 
-sdp_uuid_nam_lookup_table_t sdp_uuid_nam_lookup_table[] = {
+static sdp_uuid_nam_lookup_table_t sdp_uuid_nam_lookup_table[] = {
 	{ SDP_UUID_SDP,                      "SDP"          },
 	{ SDP_UUID_UDP,                      "UDP"          },
 	{ SDP_UUID_RFCOMM,                   "RFCOMM"       },
@@ -73,6 +61,9 @@ sdp_uuid_nam_lookup_table_t sdp_uuid_nam_lookup_table[] = {
 	{ SDP_UUID_WSP,                      "WSP"          },
 	{ SDP_UUID_L2CAP,                    "L2CAP"        },
 	{ SDP_UUID_BNEP,                     "BNEP"         }, /* PAN */
+	{ SDP_UUID_HIDP,                     "HIDP"         }, /* HID */
+	{ SDP_UUID_AVDTP,                    "AVDTP"        }, /* AVDTP */
+	{ SDP_UUID_CMTP,                     "CMTP"         }, /* CIP */
 	{ SDP_UUID_SERVICE_DISCOVERY_SERVER, "SDServer"     },
 	{ SDP_UUID_BROWSE_GROUP_DESCRIPTOR,  "BrwsGrpDesc"  },
 	{ SDP_UUID_PUBLIC_BROWSE_GROUP,      "PubBrwsGrp"   },
@@ -85,21 +76,32 @@ sdp_uuid_nam_lookup_table_t sdp_uuid_nam_lookup_table[] = {
 	{ SDP_UUID_IR_MC_SYNC_COMMAND,       "IRMCSyncCmd"  },
 	{ SDP_UUID_HEADSET,                  "Headset"      },
 	{ SDP_UUID_CORDLESS_TELEPHONY,       "CordlessTel"  },
+	{ SDP_UUID_AUDIO_SOURCE,             "AudioSource"  }, /* A2DP */
+	{ SDP_UUID_AUDIO_SINK,               "AudioSink"    }, /* A2DP */
+	{ SDP_UUID_ADVANCED_AUDIO,           "AdvAudio"     }, /* A2DP */
 	{ SDP_UUID_INTERCOM,                 "Intercom"     },
 	{ SDP_UUID_FAX,                      "Fax"          },
 	{ SDP_UUID_HEADSET_AUDIO_GATEWAY,    "AG"           },
 	{ SDP_UUID_PANU,                     "PANU"         }, /* PAN */
 	{ SDP_UUID_NAP,                      "NAP"          }, /* PAN */
 	{ SDP_UUID_GN,                       "GN"           }, /* PAN */
-	{ SDP_UUID_CMTP,                     "CMTP"         }, /* CIP */
+	{ SDP_UUID_IMAGING,                  "Imaging"      }, /* BIP */
+	{ SDP_UUID_IMAGING_RESPONDER,        "ImagingResp"  }, /* BIP */
+	{ SDP_UUID_HUMAN_INTERFACE_DEVICE,   "HID"          }, /* HID */
+	{ SDP_UUID_HARDCOPY_CABLE_REPLACE,   "HCRP",        }, /* HCRP */
+	{ SDP_UUID_HCR_PRINT,                "HCRPrint",    }, /* HCRP */
+	{ SDP_UUID_HCR_SCAN,                 "HCRScan",     }, /* HCRP */
+	{ SDP_UUID_COMMON_ISDN_ACCESS,       "CIP"          }, /* CIP */
 	{ SDP_UUID_PNP_INFORMATION,          "PNPInfo"      },
 	{ SDP_UUID_GENERIC_NETWORKING,       "Networking"   },
 	{ SDP_UUID_GENERIC_FILE_TRANSFER,    "FileTrnsf"    },
 	{ SDP_UUID_GENERIC_AUDIO,            "Audio"        },
-	{ SDP_UUID_GENERIC_TELEPHONY,        "Telephony"    }
+	{ SDP_UUID_GENERIC_TELEPHONY,        "Telephony"    },
+	{ SDP_UUID_VIDEO_SOURCE,             "VideoSource"  }, /* VDP */
+	{ SDP_UUID_VIDEO_SINK,               "VideoSink"    }  /* VDP */
 };
 
-sdp_attr_id_nam_lookup_table_t sdp_attr_id_nam_lookup_table[] = {
+static sdp_attr_id_nam_lookup_table_t sdp_attr_id_nam_lookup_table[] = {
 	{ SDP_ATTR_ID_SERVICE_RECORD_HANDLE,             "SrvRecHndl"         },
 	{ SDP_ATTR_ID_SERVICE_CLASS_ID_LIST,             "SrvClassIDList"     },
 	{ SDP_ATTR_ID_SERVICE_RECORD_STATE,              "SrvRecState"        },
@@ -122,6 +124,8 @@ sdp_attr_id_nam_lookup_table_t sdp_attr_id_nam_lookup_table[] = {
 	{ SDP_ATTR_ID_SERVICE_DATABASE_STATE,            "SrvDBState"         },
 	{ SDP_ATTR_ID_SERVICE_VERSION,                   "SrvVersion"         },
 	{ SDP_ATTR_ID_SECURITY_DESCRIPTION,              "SecurityDescription"}, /* PAN */
+	{ SDP_ATTR_ID_SUPPORTED_DATA_STORES_LIST,        "SuppDataStoresList" }, /* Synchronization */
+	{ SDP_ATTR_ID_SUPPORTED_FORMATS_LIST,            "SuppFormatsList"    }, /* OBEX Object Push */
 	{ SDP_ATTR_ID_NET_ACCESS_TYPE,                   "NetAccessType"      }, /* PAN */
 	{ SDP_ATTR_ID_MAX_NET_ACCESS_RATE,               "MaxNetAccessRate"   }, /* PAN */
 	{ SDP_ATTR_ID_IPV4_SUBNET,                       "IPv4Subnet"         }, /* PAN */
@@ -150,11 +154,11 @@ static inline char* get_attr_id_name(int attr_id)
 	return 0;
 }
 
-static inline __u8 parse_de_hdr(struct frame *frm, int* n)
+static inline uint8_t parse_de_hdr(struct frame *frm, int *n)
 {
-	__u8	de_hdr = get_u8(frm);
-	__u8	de_type = de_hdr >> 3;
-	__u8	siz_idx = de_hdr & 0x07;
+	uint8_t de_hdr = get_u8(frm);
+	uint8_t de_type = de_hdr >> 3;
+	uint8_t siz_idx = de_hdr & 0x07;
 
 	/* Get the number of bytes */
 	if (sdp_siz_idx_lookup_table[siz_idx].addl_bits) {
@@ -175,9 +179,9 @@ static inline __u8 parse_de_hdr(struct frame *frm, int* n)
 	return de_type;
 }
 
-static inline void print_int(__u8 de_type, int level, int n, struct frame *frm)
+static inline void print_int(uint8_t de_type, int level, int n, struct frame *frm, uint16_t *psm, uint8_t *channel)
 {
-	__u64 val, val2;
+	uint64_t val, val2;
 
 	switch(de_type) {
 	case SDP_DE_UINT:
@@ -194,9 +198,15 @@ static inline void print_int(__u8 de_type, int level, int n, struct frame *frm)
 	switch(n) {
 	case 1: /* 8-bit */
 		val = get_u8(frm);
+		if (channel && de_type == SDP_DE_UINT)
+			if (*channel == 0)
+				*channel = val;
 		break;
 	case 2: /* 16-bit */
 		val = get_u16(frm);
+		if (psm && de_type == SDP_DE_UINT)
+			if (*psm == 0)
+				*psm = val;
 		break;
 	case 4: /* 32-bit */
 		val = get_u32(frm);
@@ -207,7 +217,7 @@ static inline void print_int(__u8 de_type, int level, int n, struct frame *frm)
 	case 16:/* 128-bit */
 		get_u128(frm, &val, &val2);
 		printf(" 0x%llx", val2);
-		if (val < 0x1000000000000000L)
+		if (val < 0x1000000000000000LL)
 			printf("0");
 		printf("%llx", val);
 		return;
@@ -221,9 +231,9 @@ static inline void print_int(__u8 de_type, int level, int n, struct frame *frm)
 	printf(" 0x%llx", val);
 }
 
-static inline void print_uuid(int n, struct frame *frm)
+static inline void print_uuid(int n, struct frame *frm, uint16_t *psm, uint8_t *channel)
 {
-	__u32 uuid = 0;
+	uint32_t uuid = 0;
 	char* s;
 
 	switch(n) {
@@ -248,6 +258,16 @@ static inline void print_uuid(int n, struct frame *frm)
 		return;
 	}
 
+	if (psm && *psm > 0 && *psm != 0xffff) {
+		set_proto(frm->handle, *psm, 0, uuid);
+		*psm = 0xffff;
+	}
+
+	if (channel && *channel > 0 && *channel != 0xff) {
+		set_proto(frm->handle, *psm, *channel, uuid);
+		*channel = 0xff;
+	}
+
 	printf(" %s 0x%04x", s, uuid);
 	if ((s = get_uuid_name(uuid)))
 		printf(" (%s)", s);
@@ -259,7 +279,7 @@ static inline void print_string(int n, struct frame *frm, const char *name)
 
 	printf(" %s", name);
 	if ((s = malloc(n + 1))) {
-	        strncpy(s, frm->ptr, n);
+		strncpy(s, frm->ptr, n);
 		s[n] = '\0';
 		printf(" \"%s\"", s);
 		free(s);
@@ -270,63 +290,70 @@ static inline void print_string(int n, struct frame *frm, const char *name)
 	frm->len -= n;
 }
 
-static inline void print_des(__u8 de_type, int level, int n, struct frame *frm, int *split)
+static inline void print_de(int, struct frame *frm, int *split, uint16_t *psm, uint8_t *channel);
+
+static inline void print_des(uint8_t de_type, int level, int n, struct frame *frm, int *split, uint16_t *psm, uint8_t *channel)
 {
 	int len = frm->len;
-	while (len - frm->len < n )
-		print_de(level, frm, split);
+	while (len - frm->len < n && frm->len > 0)
+		print_de(level, frm, split, psm, channel);
 }
 
-static inline void print_de(int level, struct frame *frm, int *split)
+static inline void print_de(int level, struct frame *frm, int *split, uint16_t *psm, uint8_t *channel)
 {
-	int  n;
-	__u8 de_type = parse_de_hdr(frm, &n);
+	int n;
+	uint8_t de_type = parse_de_hdr(frm, &n);
 
-	switch(de_type) {
+	switch (de_type) {
 	case SDP_DE_NULL:
-		printf("null");
+		printf(" null");
 		break;
 	case SDP_DE_UINT:
 	case SDP_DE_INT:
 	case SDP_DE_BOOL:
-		print_int(de_type, level, n, frm);
+		print_int(de_type, level, n, frm, psm, channel);
 		break;
 	case SDP_DE_UUID:
 		if (split) {
 			/* Split output by uuids.
 			 * Used for printing Protocol Desc List */
-	       		if (*split) {
+			if (*split) {
 				printf("\n");
 				p_indent(level, NULL);
 			}
 			++*split;
 		}
-		print_uuid(n, frm);
+		print_uuid(n, frm, psm, channel);
 		break;
 	case SDP_DE_URL:
 	case SDP_DE_STRING:
 		print_string(n, frm, de_type == SDP_DE_URL? "url": "str");
 		break;
 	case SDP_DE_SEQ:
+		printf(" <");
+		print_des(de_type, level, n, frm, split, psm, channel);
+		printf(" >");
+		break;
 	case SDP_DE_ALT:
-		print_des(de_type, level, n, frm, split);
+		printf(" [");
+		print_des(de_type, level, n, frm, split, psm, channel);
+		printf(" ]");
 		break;
 	}
 }
 
 static inline void print_srv_srch_pat(int level, struct frame *frm)
 {
-	int len = frm->len;
-	int n1;
-	int n2;
+	int len, n1, n2;
 
 	p_indent(level, frm);
 	printf("pat");
 
 	if (parse_de_hdr(frm, &n1) == SDP_DE_SEQ) {
-		while (len - frm->len < n1 ) {
+		len = frm->len;
+		while (len - frm->len < n1 && frm->len > 0) {
 			if (parse_de_hdr(frm,&n2) == SDP_DE_UUID) {
-				print_uuid(n2, frm);
+				print_uuid(n2, frm, NULL, NULL);
 			} else {
 				printf("\nERROR: Unexpected syntax (UUID)\n");
 				raw_dump(level, frm);
@@ -342,22 +369,26 @@ static inline void print_srv_srch_pat(int level, struct frame *frm)
 
 static inline void print_attr_id_list(int level, struct frame *frm)
 {
-	__u16 attr_id;
-	__u32 attr_id_range;
-	int len = frm->len;
-	int n1, n2;
+	uint16_t attr_id;
+	uint32_t attr_id_range;
+	int len, n1, n2;
 
 	p_indent(level, frm);
 	printf("aid(s)");
 
 	if (parse_de_hdr(frm, &n1) == SDP_DE_SEQ) {
-		while (len - frm->len <= n1 ) {
+		len = frm->len;
+		while (len - frm->len < n1 && frm->len > 0) {
 			/* Print AttributeID */
 			if (parse_de_hdr(frm, &n2) == SDP_DE_UINT) {
+				char *name;
 				switch(n2) {
 				case 2:
 					attr_id = get_u16(frm);
-					printf(" 0x%04x (%s)", attr_id, get_attr_id_name(attr_id));
+					name = get_attr_id_name(attr_id);
+					if (!name)
+						name = "unknown";
+					printf(" 0x%04x (%s)", attr_id, name);
 					break;
 				case 4:
 					attr_id_range = get_u32(frm);
@@ -380,15 +411,15 @@ static inline void print_attr_id_list(int level, struct frame *frm)
 
 static inline void print_attr_list(int level, struct frame *frm)
 {
-	__u16 attr_id;
-	int   n1, n2, split;
-	int   len = frm->len;
+	uint16_t attr_id, psm;
+	uint8_t channel;
+	int len, n1, n2, split;
 
 	if (parse_de_hdr(frm, &n1) == SDP_DE_SEQ) {
-		while (len - frm->len < n1 ) {
+		len = frm->len;
+		while (len - frm->len < n1 && frm->len > 0) {
 			/* Print AttributeID */
-			if ((parse_de_hdr(frm, &n2) == SDP_DE_UINT) &&
-				 			(n2 == sizeof(attr_id))) {
+			if (parse_de_hdr(frm, &n2) == SDP_DE_UINT && n2 == sizeof(attr_id)) {
 				char *name;
 				attr_id = get_u16(frm);
 				p_indent(level, 0);
@@ -396,15 +427,15 @@ static inline void print_attr_list(int level, struct frame *frm)
 				if (!name)
 					name = "unknown";
 				printf("aid 0x%04x (%s)\n", attr_id, name);
+				split = (attr_id != SDP_ATTR_ID_PROTOCOL_DESCRIPTOR_LIST);
+				psm = 0;
+				channel = 0;
 
-				if (attr_id == SDP_ATTR_ID_PROTOCOL_DESCRIPTOR_LIST)
-					split = 0;
-				else
-					split = 1;
-				
 				/* Print AttributeValue */
 				p_indent(level + 1, 0);
-				print_de(level + 1, frm, split ? NULL: &split);
+				print_de(level + 1, frm, split ? NULL: &split,
+					attr_id == SDP_ATTR_ID_PROTOCOL_DESCRIPTOR_LIST ? &psm : NULL,
+					attr_id == SDP_ATTR_ID_PROTOCOL_DESCRIPTOR_LIST ? &channel : NULL);
 				printf("\n");
 			} else {
 				printf("\nERROR: Unexpected syntax\n");
@@ -422,12 +453,10 @@ static inline void print_attr_list(int level, struct frame *frm)
 
 static inline void print_attr_lists(int level, struct frame *frm, int len)
 {
-	int   n;
-	int   cnt = 0;
+	int n, cnt = 0;
 
 	if (parse_de_hdr(frm, &n) == SDP_DE_SEQ) {
-	printf(" len 0x%x frm->len 0x%x n 0x%x\n", len, frm->len, n);
-		while (len - frm->len < n ) {
+		while (len - frm->len < n && frm->len > 0) {
 			p_indent(level, 0);
 			printf("srv rec #%d\n", cnt++);
 			print_attr_list(level+1, frm);
@@ -439,7 +468,7 @@ static inline void print_attr_lists(int level, struct frame *frm, int len)
 }
 
 
-static inline void err_rsp(int level, __u16 tid, __u16 len, struct frame *frm)
+static inline void err_rsp(int level, uint16_t tid, uint16_t len, struct frame *frm)
 {
 	printf("SDP Error Rsp: tid 0x%x len 0x%x\n", tid, len);
 
@@ -453,7 +482,7 @@ static inline void err_rsp(int level, __u16 tid, __u16 len, struct frame *frm)
 }
 
 
-static inline void ss_req(int level, __u16 tid, __u16 len, struct frame *frm)
+static inline void ss_req(int level, uint16_t tid, uint16_t len, struct frame *frm)
 {
 	printf("SDP SS Req: tid 0x%x len 0x%x\n", tid, len);
 
@@ -465,16 +494,16 @@ static inline void ss_req(int level, __u16 tid, __u16 len, struct frame *frm)
 	printf("max 0x%x\n", get_u16(frm));
 }
 
-static inline void ss_rsp(int level, __u16 tid, __u16 len, struct frame *frm)
+static inline void ss_rsp(int level, uint16_t tid, uint16_t len, struct frame *frm)
 {
 	register int i;
-	__u16 cur_srv_rec_cnt = get_u16(frm); /* Parse CurrentServiceRecordCount */
-	__u16 tot_srv_rec_cnt = get_u16(frm); /* Parse TotalServiceRecordCount */
+	uint16_t tot_srv_rec_cnt = get_u16(frm); /* Parse TotalServiceRecordCount */
+	uint16_t cur_srv_rec_cnt = get_u16(frm); /* Parse CurrentServiceRecordCount */
 
 	printf("SDP SS Rsp: tid 0x%x len 0x%x\n", tid, len);
 
 	p_indent(++level, 0);
-	printf("cur 0x%x tot 0x%x", cur_srv_rec_cnt, tot_srv_rec_cnt);
+	printf("tot 0x%x cur 0x%x", tot_srv_rec_cnt, cur_srv_rec_cnt);
 
 	/* Parse service record handle(s) */
 	if (cur_srv_rec_cnt > 0) {
@@ -486,7 +515,7 @@ static inline void ss_rsp(int level, __u16 tid, __u16 len, struct frame *frm)
 	printf("\n");
 }
 
-static inline void sa_req(int level, __u16 tid, __u16 len, struct frame *frm)
+static inline void sa_req(int level, uint16_t tid, uint16_t len, struct frame *frm)
 {
 	printf("SDP SA Req: tid 0x%x len 0x%x\n", tid, len);
 
@@ -502,7 +531,7 @@ static inline void sa_req(int level, __u16 tid, __u16 len, struct frame *frm)
 	print_attr_id_list(level, frm);
 }
 
-static inline void sa_rsp(int level, __u16 tid, __u16 len, struct frame *frm)
+static inline void sa_rsp(int level, uint16_t tid, uint16_t len, struct frame *frm)
 {
 	printf("SDP SA Rsp: tid 0x%x len 0x%x\n", tid, len);
 
@@ -514,7 +543,7 @@ static inline void sa_rsp(int level, __u16 tid, __u16 len, struct frame *frm)
 	print_attr_list(level, frm);
 }
 
-static inline void ssa_req(int level, __u16 tid, __u16 len, struct frame *frm)
+static inline void ssa_req(int level, uint16_t tid, uint16_t len, struct frame *frm)
 {
 	printf("SDP SSA Req: tid 0x%x len 0x%x\n", tid, len);
 
@@ -529,7 +558,7 @@ static inline void ssa_req(int level, __u16 tid, __u16 len, struct frame *frm)
 	print_attr_id_list(level, frm);
 }
 
-static inline void ssa_rsp(int level, __u16 tid, __u16 len, struct frame *frm)
+static inline void ssa_rsp(int level, uint16_t tid, uint16_t len, struct frame *frm)
 {
 	int cnt;
 	printf("SDP SSA Rsp: tid 0x%x len 0x%x\n", tid, len);
@@ -546,8 +575,8 @@ static inline void ssa_rsp(int level, __u16 tid, __u16 len, struct frame *frm)
 void sdp_dump(int level, struct frame *frm)
 {
 	sdp_pdu_hdr *hdr = frm->ptr;
- 	__u16 tid = ntohs(hdr->tid);
-	__u16 len = ntohs(hdr->len);
+ 	uint16_t tid = ntohs(hdr->tid);
+	uint16_t len = ntohs(hdr->len);
 
 	frm->ptr += SDP_PDU_HDR_SIZE;
 	frm->len -= SDP_PDU_HDR_SIZE;
